@@ -27,10 +27,14 @@ from utils import Dirs, SystemSettings
 from widgets import DigitalClockDrawing, SelectableIconView, ContentView
 
 
+# keep the GWeather world around as a singletom, otherwise
+# if is garbage collected get_city_name etc fail.
+gweather_world = GWeather.Location.new_world(True)
+
+
 class WorldClockStorage():
     def __init__(self):
         self.filename = os.path.join(Dirs.get_user_data_dir(), "clocks.json")
-        self.world = GWeather.Location.new_world(True)
 
     def save(self, clocks):
         location_codes = [c.location.get_code() for c in clocks]
@@ -45,7 +49,7 @@ class WorldClockStorage():
             location_codes = json.load(f)
             f.close()
             for l in location_codes:
-                location = GWeather.Location.find_by_station_code(self.world, l)
+                location = GWeather.Location.find_by_station_code(gweather_world, l)
                 if location:
                     clock = DigitalClock(location)
                     clocks.append(clock)
@@ -71,8 +75,7 @@ class NewWorldClockDialog(Gtk.Dialog):
         label = Gtk.Label(_("Search for a city:"))
         label.set_alignment(0.0, 0.5)
 
-        world = GWeather.Location.new_world(True)
-        self.entry = GWeather.LocationEntry.new(world)
+        self.entry = GWeather.LocationEntry.new(gweather_world)
         self.find_gicon = Gio.ThemedIcon.new_with_default_fallbacks(
             'edit-find-symbolic')
         self.clear_gicon = Gio.ThemedIcon.new_with_default_fallbacks(
@@ -131,40 +134,51 @@ class DigitalClock():
         self.path = None
         self.list_store = None
 
-        timezone = self.location.get_timezone()
-        self.offset = timezone.get_offset() * 60
+        weather_timezone = self.location.get_timezone()
+        timezone = GLib.TimeZone.new(weather_timezone.get_tzid())
+        i = timezone.find_interval(GLib.TimeType.UNIVERSAL, time.time())
+        location_offset = timezone.get_offset(i)
+
+        timezone = GLib.TimeZone.new_local()
+        i = timezone.find_interval(GLib.TimeType.UNIVERSAL, time.time())
+        here_offset = timezone.get_offset(i)
+
+        self.offset = location_offset - here_offset
+
         self._last_time = None
         self.drawing = DigitalClockDrawing()
         self.standalone = None
         self.update()
 
-    def get_local_time(self, secs):
-        t = secs + time.timezone + self.offset
+    def get_location_time(self, secs=None):
+        if not secs:
+            secs = time.time()
+        t = secs + self.offset
         t = time.localtime(t)
         return t
 
     def update(self):
-        systemClockFormat = SystemSettings.get_clock_format()
-        if systemClockFormat == '12h':
-            t = time.strftime("%I:%M %p", self.get_local_time(time.time()))
+        clock_format = SystemSettings.get_clock_format()
+        location_time = self.get_location_time()
+        if clock_format == '12h':
+            t = time.strftime("%I:%M %p", location_time)
         else:
-            t = time.strftime("%H:%M", self.get_local_time(time.time()))
+            t = time.strftime("%H:%M", location_time)
         if t.startswith("0"):
             t = t[1:]
         if not t == self._last_time \
                 or not self.sunrise == self._last_sunrise \
                 or not self.sunset == self._last_sunset:
-            local_time = self.get_local_time(time.time())
-            isDay = self.get_is_light(local_time, self.sunrise, self.sunset)
-            if isDay:
+            is_light = self.get_is_light(location_time)
+            if is_light:
                 img = os.path.join(Dirs.get_image_dir(), "cities", "day.png")
             else:
                 img = os.path.join(Dirs.get_image_dir(), "cities", "night.png")
             day = self.get_day()
             if day == "Today":
-                self.drawing.render(t, img, isDay)
+                self.drawing.render(t, img, is_light)
             else:
-                self.drawing.render(t, img, isDay, day)
+                self.drawing.render(t, img, is_light, day)
             if self.path and self.list_store:
                 self.list_store[self.path][1] = self.drawing.pixbuf
             if self.standalone:
@@ -173,8 +187,7 @@ class DigitalClock():
         self._last_time = t
 
     def get_sunrise_sunset(self):
-        world = GWeather.Location.new_world(True)
-        self.weather = GWeather.Info(location=self.location, world=world)
+        self.weather = GWeather.Info(location=self.location, world=gweather_world)
         self.weather.connect('updated', self._on_weather_updated)
         self.weather.update()
 
@@ -184,8 +197,8 @@ class DigitalClock():
         ok, sunset = weather.get_value_sunset()
         self._last_sunrise = self.sunrise
         self._last_sunset = self.sunset
-        self.sunrise = self.get_local_time(sunrise)
-        self.sunset = self.get_local_time(sunset)
+        self.sunrise = self.get_location_time(sunrise)
+        self.sunset = self.get_location_time(sunset)
         self.update()
 
     def get_pixbuf(self):
@@ -198,7 +211,7 @@ class DigitalClock():
         return self.standalone
 
     def get_day(self):
-        clock_time_day = self.get_local_time(time.time()).tm_yday
+        clock_time_day = self.get_location_time().tm_yday
         local_time_day = time.localtime().tm_yday
 
         if clock_time_day == local_time_day:
@@ -218,20 +231,20 @@ class DigitalClock():
             else:
                 return "Yesterday"
 
-    def get_is_light(self, current, sunrise, sunset):
-        if current.tm_hour < sunrise.tm_hour \
-                or current.tm_hour > sunset.tm_hour:
+    def get_is_light(self, current):
+        if current.tm_hour < self.sunrise.tm_hour \
+                or current.tm_hour > self.sunset.tm_hour:
             return False
-        elif current.tm_hour > sunrise.tm_hour \
-                and current.tm_hour < sunset.tm_hour:
+        elif current.tm_hour > self.sunrise.tm_hour \
+                and current.tm_hour < self.sunset.tm_hour:
             return True
-        elif current.tm_hour == sunrise.tm_hour:
-            if current.tm_min >= sunrise.tm_min:
+        elif current.tm_hour == self.sunrise.tm_hour:
+            if current.tm_min >= self.sunrise.tm_min:
                 return True
             else:
                 return False
-        elif current.tm_hour == sunset.tm_hour:
-            if current.tm_min <= sunrise.tm_min:
+        elif current.tm_hour == self.sunset.tm_hour:
+            if current.tm_min <= self.sunrise.tm_min:
                 return True
             else:
                 return False
@@ -344,42 +357,36 @@ class World(Clock):
         dialog.destroy()
 
 
-class StandaloneClock(Gtk.Box):
+class StandaloneClock(Gtk.EventBox):
     def __init__(self, location, sunrise, sunset):
-        Gtk.Box.__init__(self, orientation=Gtk.Orientation.VERTICAL)
+        Gtk.EventBox.__init__(self)
+        self.get_style_context().add_class('view')
+        self.get_style_context().add_class('content-view')
         self.location = location
         self.can_edit = False
-        #self.img = Gtk.Image()
-        #self.city_label = Gtk.Label()
-        #label = GLib.markup_escape_text(location.get_city_name())
-        #self.city_label.set_markup("<b>%s</b>" % label)
         self.time_label = Gtk.Label()
         self.sunrise = sunrise
         self.sunset = sunset
 
-        self.systemClockFormat = None
+        self.clock_format = None
 
-        #imagebox = Gtk.VBox()
-        #imagebox.pack_start(self.img, False, False, 0)
-        #imagebox.pack_start(self.city_label, False, False, 0)
-        #imagebox.set_size_request(230, 230)
+        self.vbox = Gtk.Box(orientation=Gtk.Orientation.VERTICAL)
+        self.add(self.vbox)
 
-        self.timebox = Gtk.Box(orientation=Gtk.Orientation.VERTICAL)
+        time_box = Gtk.Box(orientation=Gtk.Orientation.VERTICAL)
         self.time_label.set_alignment(0.0, 0.5)
-        self.timebox.pack_start(self.time_label, True, True, 0)
+        time_box.pack_start(self.time_label, True, True, 0)
 
         self.hbox = hbox = Gtk.Box()
         self.hbox.set_homogeneous(False)
 
         self.hbox.pack_start(Gtk.Label(), True, True, 0)
-        # self.hbox.pack_start(imagebox, False, False, 0)
-        # self.hbox.pack_start(Gtk.Label(), False, False, 30)
-        self.hbox.pack_start(self.timebox, False, False, 0)
+        self.hbox.pack_start(time_box, False, False, 0)
         self.hbox.pack_start(Gtk.Label(), True, True, 0)
 
-        self.pack_start(Gtk.Label(), True, True, 25)
-        self.pack_start(hbox, False, False, 0)
-        self.pack_start(Gtk.Label(), True, True, 0)
+        self.vbox.pack_start(Gtk.Label(), True, True, 25)
+        self.vbox.pack_start(hbox, False, False, 0)
+        self.vbox.pack_start(Gtk.Label(), True, True, 0)
 
         sunrise_label = Gtk.Label()
         sunrise_label.set_markup(
@@ -411,7 +418,7 @@ class StandaloneClock(Gtk.Box):
         hbox.pack_start(Gtk.Label(), True, True, 0)
         hbox.pack_start(sunbox, False, False, 0)
         hbox.pack_start(Gtk.Label(), True, True, 0)
-        self.pack_end(hbox, False, False, 30)
+        self.vbox.pack_end(hbox, False, False, 30)
 
         self.show_all()
 
@@ -420,18 +427,14 @@ class StandaloneClock(Gtk.Box):
 
     def update(self, img, text, sunrise, sunset):
         size = 72000  # FIXME: (self.get_allocation().height / 300) * 72000
-        #if img:
-        #    pixbuf = GdkPixbuf.Pixbuf.new_from_file_at_size(img, 500, 380)
-        #    pixbuf = pixbuf.new_subpixbuf(0, 0, 208, 208)
-        #    self.img.set_from_pixbuf(pixbuf)
         self.time_label.set_markup(
             "<span size='%i' color='dimgray'><b>%s</b></span>" % (size, text))
-        systemClockFormat = SystemSettings.get_clock_format()
-        if systemClockFormat != self.systemClockFormat or \
+        clock_format = SystemSettings.get_clock_format()
+        if clock_format != self.clock_format or \
                 sunrise != self.sunrise or sunset != self.sunset:
             self.sunrise = sunrise
             self.sunset = sunset
-            if systemClockFormat == "12h":
+            if clock_format == "12h":
                 sunrise_str = time.strftime("%I:%M %p", sunrise)
                 sunset_str = time.strftime("%I:%M %p", sunset)
             else:
@@ -445,4 +448,4 @@ class StandaloneClock(Gtk.Box):
                 "<span size ='large'>%s</span>" % sunrise_str)
             self.sunset_time_label.set_markup(
                 "<span size ='large'>%s</span>" % sunset_str)
-        self.systemClockFormat = systemClockFormat
+        self.clock_format = clock_format
